
import numpy as np
import pandas as pd

from pymatgen.core import Structure, Lattice
from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
import os
import re
sg_encoding= {'Aba2': {'enc': '03aODDbOOOjDDO0', 'full_symbol': 'Aea2', 'int_number': 41, 'order': 8, 'patterson_symmetry': 'Ammm (Cmmm)', 'point_group': 'mm2'}, 'Abm2': {'enc': '03aODDbOOOjODO0', 'full_symbol': 'Aem2', 'int_number': 39, 'order': 8, 'patterson_symmetry': 'Ammm (Cmmm)', 'point_group': 'mm2'}, 'Aea2': {'enc': '03aODDbOOOjDDO0', 'full_symbol': 'Aea2', 'int_number': 41, 'order': 8, 'patterson_symmetry': 'Ammm (Cmmm)', 'point_group': 'mm2'}, 'Aem2': {'enc': '03aODDbOOOjODO0', 'full_symbol': 'Aem2', 'int_number': 39, 'order': 8, 'patterson_symmetry': 'Ammm (Cmmm)', 'point_group': 'mm2'}, 'Ama2': {'enc': '03aODDbOOOjDOO0', 'full_symbol': 'Ama2', 'int_number': 40, 'order': 8, 'patterson_symmetry': 'Ammm (Cmmm)', 'point_group': 'mm2'}, 'Amm2': {'enc': '03aODDbOOOjOOO0', 'full_symbol': 'Amm2', 'int_number': 38, 'order': 8, 'patterson_symmetry': 'Ammm (Cmmm)', 'point_group': 'mm2'}, 'C12/c1': {'enc': '12aDDOcOOD0', 'full_symbol': 'C12/c1', 'int_number': 15, 'order': 8, 'patterson_symmetry': 'C12/m1', 'point_group': '2/m'}, 'C12/m1': {'enc': '12aDDOcOOO0', 'full_symbol': 'C12/m1', 'int_number': 12, 'order': 8, 'patterson_symmetry': 'C12/m1', 'point_group': '2/m'}, 'C121': {'enc': '02aDDOcOOO0', 'full_symbol': 'C121', 'int_number': 5, 'order': 4, 'patterson_symmetry': 'C12/m1', 'point_group': '2'}, 'C1c1': {'enc': '02aDDOjOOD0', 'full_symbol': 'C1c1', 'int_number': 9, 'order': 4, 'patterson_symmetry': 'C12/m1', 'point_group': 'm'}, 'C1m1': {'enc': '02aDDOjOOO0', 'full_symbol': 'C1m1', 'int_number': 8, 'order': 4, 'patterson_symmetry': 'C12/m1', 'point_group': 'm'}, 'C222': {'enc': '03aDDObOOOcOOO0', 'full_symbol': 'C222', 'int_number': 21, 'order': 8, 'patterson_symmetry': 'Cmmm', 'point_group': '222'}, 'C222_1': {'enc': '03aDDObOODcOOD0', 'full_symbol': 'C222_1', 'int_number': 20, 'order': 8, 'patterson_symmetry': 'Cmmm', 'point_group': '222'}, 'Ccc2': {'enc': '03aDDObOOOjOOD0', 'full_symbol': 'Ccc2', 'int_number': 37, 'order': 8, 'patterson_symmetry': 'Cmmm', 'point_group': 'mm2'}, 'Ccca': {'enc': '04aDDObDDOcOOOhODD1OBB', 'full_symbol': 'C2/c2/c2/e', 'int_number': 68, 'order': 16, 'patterson_symmetry': 'Cmmm', 'point_group': 'mmm'}, 'Ccce': {'enc': '04aDDObDDOcOOOhODD1OBB', 'full_symbol': 'C2/c2/c2/e', 'int_number': 68, 'order': 16, 'patterson_symmetry': 'Cmmm', 'point_group': 'mmm'}, 'Cccm': {'enc': '13aDDObOOOcOOD0', 'full_symbol': 'C2/c2/c2/m', 'int_number': 66, 'order': 16, 'patterson_symmetry': 'Cmmm', 'point_group': 'mmm'}, 'Cmc2_1': {'enc': '03aDDObOODjOOD0', 'full_symbol': 'Cmc2_1', 'int_number': 36, 'order': 8, 'patterson_symmetry': 'Cmmm', 'point_group': 'mm2'}, 'Cmce': {'enc': '13aDDObODDcODD0', 'full_symbol': 'C2/m2/c2_1/e', 'int_number': 64, 'order': 16, 'patterson_symmetry': 'Cmmm', 'point_group': 'mmm'}, 'Cmcm': {'enc': '13aDDObOODcOOD0', 'full_symbol': 'C2/m2/c2_1/m', 'int_number': 63, 'order': 16, 'patterson_symmetry': 'Cmmm', 'point_group': 'mmm'}, 'Cmm2': {'enc': '03aDDObOOOjOOO0', 'full_symbol': 'Cmm2', 'int_number': 35, 'order': 8, 'patterson_symmetry': 'Cmmm', 'point_group': 'mm2'}, 'Cmma': {'enc': '13aDDObODOcODO0', 'full_symbol': 'C2/m2/m2/e', 'int_number': 67, 'order': 16, 'patterson_symmetry': 'Cmmm', 'point_group': 'mmm'}, 'Cmme': {'enc': '13aDDObODOcODO0', 'full_symbol': 'C2/m2/m2/e', 'int_number': 67, 'order': 16, 'patterson_symmetry': 'Cmmm', 'point_group': 'mmm'}, 'Cmmm': {'enc': '13aDDObOOOcOOO0', 'full_symbol': 'C2/m2/m2/m', 'int_number': 65, 'order': 16, 'patterson_symmetry': 'Cmmm', 'point_group': 'mmm'}, 'F-43c': {'enc': '06aODDaDODbOOOcOOOdOOOlDDD0', 'full_symbol': 'F-43c', 'int_number': 219, 'order': 96, 'patterson_symmetry': 'Fm-3m', 'point_group': '-43m'}, 'F-43m': {'enc': '06aODDaDODbOOOcOOOdOOOlOOO0', 'full_symbol': 'F-43m', 'int_number': 216, 'order': 96, 'patterson_symmetry': 'Fm-3m', 'point_group': '-43m'}, 'F222': {'enc': '04aODDaDODbOOOcOOO0', 'full_symbol': 'F222', 'int_number': 22, 'order': 16, 'patterson_symmetry': 'Fmmm', 'point_group': '222'}, 'F23': {'enc': '05aODDaDODbOOOcOOOdOOO0', 'full_symbol': 'F23', 'int_number': 196, 'order': 48, 'patterson_symmetry': 'Fm-3', 'point_group': '23'}, 'F432': {'enc': '06aODDaDODbOOOcOOOdOOOeOOO0', 'full_symbol': 'F432', 'int_number': 209, 'order': 96, 'patterson_symmetry': 'Fm-3m', 'point_group': '432'}, 'F4_132': {'enc': '06aODDaDODbODDcDDOdOOOeFBF0', 'full_symbol': 'F4_132', 'int_number': 210, 'order': 96, 'patterson_symmetry': 'Fm-3m', 'point_group': '432'}, 'Fd-3': {'enc': '06aODDaDODbOOOcOOOdOOOhBBB1ZZZ', 'full_symbol': 'F2/d-3', 'int_number': 203, 'order': 96, 'patterson_symmetry': 'Fm-3', 'point_group': 'm-3'}, 'Fd-3c': {'enc': '07aODDaDODbODDcDDOdOOOeFBFhFFF1XXX', 'full_symbol': 'F4_1/d-32/c', 'int_number': 228, 'order': 192, 'patterson_symmetry': 'Fm-3m', 'point_group': 'm-3m'}, 'Fd-3m': {'enc': '07aODDaDODbODDcDDOdOOOeFBFhBBB1ZZZ', 'full_symbol': 'F4_1/d-32/m', 'int_number': 227, 'order': 192, 'patterson_symmetry': 'Fm-3m', 'point_group': 'm-3m'}, 'Fdd2': {'enc': '04aODDaDODbOOOjBBB0', 'full_symbol': 'Fdd2', 'int_number': 43, 'order': 16, 'patterson_symmetry': 'Fmmm', 'point_group': 'mm2'}, 'Fddd': {'enc': '05aODDaDODbOOOcOOOhBBB1ZZZ', 'full_symbol': 'F2/d2/d2/d', 'int_number': 70, 'order': 32, 'patterson_symmetry': 'Fmmm', 'point_group': 'mmm'}, 'Fm-3': {'enc': '15aODDaDODbOOOcOOOdOOO0', 'full_symbol': 'F2/m-3', 'int_number': 202, 'order': 96, 'patterson_symmetry': 'Fm-3', 'point_group': 'm-3'}, 'Fm-3c': {'enc': '16aODDaDODbOOOcOOOdOOOeDDD0', 'full_symbol': 'F4/m-32/c', 'int_number': 226, 'order': 192, 'patterson_symmetry': 'Fm-3m', 'point_group': 'm-3m'}, 'Fm-3m': {'enc': '16aODDaDODbOOOcOOOdOOOeOOO0', 'full_symbol': 'F4/m-32/m', 'int_number': 225, 'order': 192, 'patterson_symmetry': 'Fm-3m', 'point_group': 'm-3m'}, 'Fmm2': {'enc': '04aODDaDODbOOOjOOO0', 'full_symbol': 'Fmm2', 'int_number': 42, 'order': 16, 'patterson_symmetry': 'Fmmm', 'point_group': 'mm2'}, 'Fmmm': {'enc': '14aODDaDODbOOOcOOO0', 'full_symbol': 'F2/m2/m2/m', 'int_number': 69, 'order': 32, 'patterson_symmetry': 'Fmmm', 'point_group': 'mmm'}, 'I-4': {'enc': '03aDDDbOOOmOOO0', 'full_symbol': 'I-4', 'int_number': 82, 'order': 8, 'patterson_symmetry': 'I4/m', 'point_group': '-4'}, 'I-42d': {'enc': '04aDDDbOOOmOOOcDOF0', 'full_symbol': 'I-42d', 'int_number': 122, 'order': 16, 'patterson_symmetry': 'I4/mmm', 'point_group': '-42m'}, 'I-42m': {'enc': '04aDDDbOOOmOOOcOOO0', 'full_symbol': 'I-42m', 'int_number': 121, 'order': 16, 'patterson_symmetry': 'I4/mmm', 'point_group': '-42m'}, 'I-43d': {'enc': '05aDDDbDODcODDdOOOlBBB0', 'full_symbol': 'I-43d', 'int_number': 220, 'order': 48, 'patterson_symmetry': 'Im-3m', 'point_group': '-43m'}, 'I-43m': {'enc': '05aDDDbOOOcOOOdOOOlOOO0', 'full_symbol': 'I-43m', 'int_number': 217, 'order': 48, 'patterson_symmetry': 'Im-3m', 'point_group': '-43m'}, 'I-4c2': {'enc': '04aDDDbOOOmOOOjOOD0', 'full_symbol': 'I-4c2', 'int_number': 120, 'order': 16, 'patterson_symmetry': 'I4/mmm', 'point_group': '-42m'}, 'I-4m2': {'enc': '04aDDDbOOOmOOOjOOO0', 'full_symbol': 'I-4m2', 'int_number': 119, 'order': 16, 'patterson_symmetry': 'I4/mmm', 'point_group': '-42m'}, 'I222': {'enc': '03aDDDbOOOcOOO0', 'full_symbol': 'I222', 'int_number': 23, 'order': 8, 'patterson_symmetry': 'Immm', 'point_group': '222'}, 'I23': {'enc': '04aDDDbOOOcOOOdOOO0', 'full_symbol': 'I23', 'int_number': 197, 'order': 24, 'patterson_symmetry': 'Im-3', 'point_group': '23'}, 'I2_12_12_1': {'enc': '03aDDDbDODcODD0', 'full_symbol': 'I2_12_12_1', 'int_number': 24, 'order': 8, 'patterson_symmetry': 'Immm', 'point_group': '222'}, 'I2_13': {'enc': '04aDDDbDODcODDdOOO0', 'full_symbol': 'I2_13', 'int_number': 199, 'order': 24, 'patterson_symmetry': 'Im-3', 'point_group': '23'}, 'I4': {'enc': '03aDDDbOOOgOOO0', 'full_symbol': 'I4', 'int_number': 79, 'order': 8, 'patterson_symmetry': 'I4/m', 'point_group': '4'}, 'I4/m': {'enc': '13aDDDbOOOgOOO0', 'full_symbol': 'I4/m', 'int_number': 87, 'order': 16, 'patterson_symmetry': 'I4/m', 'point_group': '4/m'}, 'I4/mcm': {'enc': '14aDDDbOOOgOOOcOOD0', 'full_symbol': 'I4/m2/c2/m', 'int_number': 140, 'order': 32, 'patterson_symmetry': 'I4/mmm', 'point_group': '4/mmm'}, 'I4/mmm': {'enc': '14aDDDbOOOgOOOcOOO0', 'full_symbol': 'I4/m2/m2/m', 'int_number': 139, 'order': 32, 'patterson_symmetry': 'I4/mmm', 'point_group': '4/mmm'}, 'I422': {'enc': '04aDDDbOOOgOOOcOOO0', 'full_symbol': 'I422', 'int_number': 97, 'order': 16, 'patterson_symmetry': 'I4/mmm', 'point_group': '422'}, 'I432': {'enc': '05aDDDbOOOcOOOdOOOeOOO0', 'full_symbol': 'I432', 'int_number': 211, 'order': 48, 'patterson_symmetry': 'Im-3m', 'point_group': '432'}, 'I4_1': {'enc': '03aDDDbDDDgODB0', 'full_symbol': 'I4_1', 'int_number': 80, 'order': 8, 'patterson_symmetry': 'I4/m', 'point_group': '4'}, 'I4_1/a': {'enc': '04aDDDbDDDgODBhODB1OYZ', 'full_symbol': 'I4_1/a', 'int_number': 88, 'order': 16, 'patterson_symmetry': 'I4/m', 'point_group': '4/m'}, 'I4_1/acd': {'enc': '05aDDDbDDDgODBcDOBhODB1OBZ', 'full_symbol': 'I4_1/a2/c2/d', 'int_number': 142, 'order': 32, 'patterson_symmetry': 'I4/mmm', 'point_group': '4/mmm'}, 'I4_1/amd': {'enc': '05aDDDbDDDgODBcDOFhODB1OBZ', 'full_symbol': 'I4_1/a2/m2/d', 'int_number': 141, 'order': 32, 'patterson_symmetry': 'I4/mmm', 'point_group': '4/mmm'}, 'I4_122': {'enc': '04aDDDbDDDgODBcDOF0', 'full_symbol': 'I4_122', 'int_number': 98, 'order': 16, 'patterson_symmetry': 'I4/mmm', 'point_group': '422'}, 'I4_132': {'enc': '05aDDDbDODcODDdOOOeFBB0', 'full_symbol': 'I4_132', 'int_number': 214, 'order': 48, 'patterson_symmetry': 'Im-3m', 'point_group': '432'}, 'I4_1cd': {'enc': '04aDDDbDDDgODBjOOD0', 'full_symbol': 'I4_1cd', 'int_number': 110, 'order': 16, 'patterson_symmetry': 'I4/mmm', 'point_group': '4mm'}, 'I4_1md': {'enc': '04aDDDbDDDgODBjOOO0', 'full_symbol': 'I4_1md', 'int_number': 109, 'order': 16, 'patterson_symmetry': 'I4/mmm', 'point_group': '4mm'}, 'I4cm': {'enc': '04aDDDbOOOgOOOjOOD0', 'full_symbol': 'I4cm', 'int_number': 108, 'order': 16, 'patterson_symmetry': 'I4/mmm', 'point_group': '4mm'}, 'I4mm': {'enc': '04aDDDbOOOgOOOjOOO0', 'full_symbol': 'I4mm', 'int_number': 107, 'order': 16, 'patterson_symmetry': 'I4/mmm', 'point_group': '4mm'}, 'Ia-3': {'enc': '14aDDDbDODcODDdOOO0', 'full_symbol': 'I2_1/a-3', 'int_number': 206, 'order': 48, 'patterson_symmetry': 'Im-3', 'point_group': 'm-3'}, 'Ia-3d': {'enc': '15aDDDbDODcODDdOOOeFBB0', 'full_symbol': 'I4_1/a-32/d', 'int_number': 230, 'order': 96, 'patterson_symmetry': 'Im-3m', 'point_group': 'm-3m'}, 'Iba2': {'enc': '03aDDDbOOOjDDO0', 'full_symbol': 'Iba2', 'int_number': 45, 'order': 8, 'patterson_symmetry': 'Immm', 'point_group': 'mm2'}, 'Ibam': {'enc': '13aDDDbOOOcDDO0', 'full_symbol': 'I2/b2/a2/m', 'int_number': 72, 'order': 16, 'patterson_symmetry': 'Immm', 'point_group': 'mmm'}, 'Ibca': {'enc': '13aDDDbDODcODD0', 'full_symbol': 'I2_1/b2_1/c2_1/a', 'int_number': 73, 'order': 16, 'patterson_symmetry': 'Immm', 'point_group': 'mmm'}, 'Im-3': {'enc': '14aDDDbOOOcOOOdOOO0', 'full_symbol': 'I2/m-3', 'int_number': 204, 'order': 48, 'patterson_symmetry': 'Im-3', 'point_group': 'm-3'}, 'Im-3m': {'enc': '15aDDDbOOOcOOOdOOOeOOO0', 'full_symbol': 'I4/m-32/m', 'int_number': 229, 'order': 96, 'patterson_symmetry': 'Im-3m', 'point_group': 'm-3m'}, 'Ima2': {'enc': '03aDDDbOOOjDOO0', 'full_symbol': 'Ima2', 'int_number': 46, 'order': 8, 'patterson_symmetry': 'Immm', 'point_group': 'mm2'}, 'Imm2': {'enc': '03aDDDbOOOjOOO0', 'full_symbol': 'Imm2', 'int_number': 44, 'order': 8, 'patterson_symmetry': 'Immm', 'point_group': 'mm2'}, 'Imma': {'enc': '13aDDDbODOcODO0', 'full_symbol': 'I2_1/m2_1/m2_1/a', 'int_number': 74, 'order': 16, 'patterson_symmetry': 'Immm', 'point_group': 'mmm'}, 'Immm': {'enc': '13aDDDbOOOcOOO0', 'full_symbol': 'I2/m2/m2/m', 'int_number': 71, 'order': 16, 'patterson_symmetry': 'Immm', 'point_group': 'mmm'}, 'P-1': {'enc': '100', 'full_symbol': 'P-1', 'int_number': 2, 'order': 2, 'patterson_symmetry': 'P-1', 'point_group': '-1'}, 'P-3': {'enc': '11nOOO0', 'full_symbol': 'P-3', 'int_number': 147, 'order': 6, 'patterson_symmetry': 'P-3', 'point_group': '-3'}, 'P-31c': {'enc': '12nOOOfOOD0', 'full_symbol': 'P-312/c', 'int_number': 163, 'order': 12, 'patterson_symmetry': 'P-31m', 'point_group': '-3m'}, 'P-31m': {'enc': '12nOOOfOOO0', 'full_symbol': 'P-312/m', 'int_number': 162, 'order': 12, 'patterson_symmetry': 'P-31m', 'point_group': '-3m'}, 'P-3c1': {'enc': '12nOOOeOOD0', 'full_symbol': 'P-32/c1', 'int_number': 165, 'order': 12, 'patterson_symmetry': 'P-3m1', 'point_group': '-3m'}, 'P-3m1': {'enc': '12nOOOeOOO0', 'full_symbol': 'P-32/m1', 'int_number': 164, 'order': 12, 'patterson_symmetry': 'P-3m1', 'point_group': '-3m'}, 'P-4': {'enc': '02bOOOmOOO0', 'full_symbol': 'P-4', 'int_number': 81, 'order': 4, 'patterson_symmetry': 'P4/m', 'point_group': '-4'}, 'P-42_1c': {'enc': '03bOOOmOOOcDDD0', 'full_symbol': 'P-42_1c', 'int_number': 114, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '-42m'}, 'P-42_1m': {'enc': '03bOOOmOOOcDDO0', 'full_symbol': 'P-42_1m', 'int_number': 113, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '-42m'}, 'P-42c': {'enc': '03bOOOmOOOcOOD0', 'full_symbol': 'P-42c', 'int_number': 112, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '-42m'}, 'P-42m': {'enc': '03bOOOmOOOcOOO0', 'full_symbol': 'P-42m', 'int_number': 111, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '-42m'}, 'P-43m': {'enc': '04bOOOcOOOdOOOlOOO0', 'full_symbol': 'P-43m', 'int_number': 215, 'order': 24, 'patterson_symmetry': 'Pm-3m', 'point_group': '-43m'}, 'P-43n': {'enc': '04bOOOcOOOdOOOlDDD0', 'full_symbol': 'P-43n', 'int_number': 218, 'order': 24, 'patterson_symmetry': 'Pm-3m', 'point_group': '-43m'}, 'P-4b2': {'enc': '03bOOOmOOOjDDO0', 'full_symbol': 'P-4b2', 'int_number': 117, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '-42m'}, 'P-4c2': {'enc': '03bOOOmOOOjOOD0', 'full_symbol': 'P-4c2', 'int_number': 116, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '-42m'}, 'P-4m2': {'enc': '03bOOOmOOOjOOO0', 'full_symbol': 'P-4m2', 'int_number': 115, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '-42m'}, 'P-4n2': {'enc': '03bOOOmOOOjDDD0', 'full_symbol': 'P-4n2', 'int_number': 118, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '-42m'}, 'P-6': {'enc': '02nOOOiOOO0', 'full_symbol': 'P-6', 'int_number': 174, 'order': 6, 'patterson_symmetry': 'P6/m', 'point_group': '-6'}, 'P-62c': {'enc': '03nOOOiOODeOOO0', 'full_symbol': 'P-62c', 'int_number': 190, 'order': 12, 'patterson_symmetry': 'P6/mmm', 'point_group': '-6m2'}, 'P-62m': {'enc': '03nOOOiOOOeOOO0', 'full_symbol': 'P-62m', 'int_number': 189, 'order': 12, 'patterson_symmetry': 'P6/mmm', 'point_group': '-6m2'}, 'P-6c2': {'enc': '03nOOOiOODkOOD0', 'full_symbol': 'P-6c2', 'int_number': 188, 'order': 12, 'patterson_symmetry': 'P6/mmm', 'point_group': '-6m2'}, 'P-6m2': {'enc': '03nOOOiOOOkOOO0', 'full_symbol': 'P-6m2', 'int_number': 187, 'order': 12, 'patterson_symmetry': 'P6/mmm', 'point_group': '-6m2'}, 'P1': {'enc': '000', 'full_symbol': 'P1', 'int_number': 1, 'order': 1, 'patterson_symmetry': 'P-1', 'point_group': '1'}, 'P12/c1': {'enc': '11cOOD0', 'full_symbol': 'P12/c1', 'int_number': 13, 'order': 4, 'patterson_symmetry': 'P12/m1', 'point_group': '2/m'}, 'P12/m1': {'enc': '11cOOO0', 'full_symbol': 'P12/m1', 'int_number': 10, 'order': 4, 'patterson_symmetry': 'P12/m1', 'point_group': '2/m'}, 'P121': {'enc': '01cOOO0', 'full_symbol': 'P121', 'int_number': 3, 'order': 2, 'patterson_symmetry': 'P12/m1', 'point_group': '2'}, 'P12_1/c1': {'enc': '11cODD0', 'full_symbol': 'P12_1/c1', 'int_number': 14, 'order': 4, 'patterson_symmetry': 'P12/m1', 'point_group': '2/m'}, 'P12_1/m1': {'enc': '11cODO0', 'full_symbol': 'P12_1/m1', 'int_number': 11, 'order': 4, 'patterson_symmetry': 'P12/m1', 'point_group': '2/m'}, 'P12_11': {'enc': '01cODO0', 'full_symbol': 'P12_11', 'int_number': 4, 'order': 2, 'patterson_symmetry': 'P12/m1', 'point_group': '2'}, 'P1c1': {'enc': '01jOOD0', 'full_symbol': 'P1c1', 'int_number': 7, 'order': 2, 'patterson_symmetry': 'P12/m1', 'point_group': 'm'}, 'P1m1': {'enc': '01jOOO0', 'full_symbol': 'P1m1', 'int_number': 6, 'order': 2, 'patterson_symmetry': 'P12/m1', 'point_group': 'm'}, 'P222': {'enc': '02bOOOcOOO0', 'full_symbol': 'P222', 'int_number': 16, 'order': 4, 'patterson_symmetry': 'Pmmm', 'point_group': '222'}, 'P222_1': {'enc': '02bOODcOOD0', 'full_symbol': 'P222_1', 'int_number': 17, 'order': 4, 'patterson_symmetry': 'Pmmm', 'point_group': '222'}, 'P23': {'enc': '03bOOOcOOOdOOO0', 'full_symbol': 'P23', 'int_number': 195, 'order': 12, 'patterson_symmetry': 'Pm-3', 'point_group': '23'}, 'P2_12_12': {'enc': '02bOOOcDDO0', 'full_symbol': 'P2_12_12', 'int_number': 18, 'order': 4, 'patterson_symmetry': 'Pmmm', 'point_group': '222'}, 'P2_12_12_1': {'enc': '02bDODcODD0', 'full_symbol': 'P2_12_12_1', 'int_number': 19, 'order': 4, 'patterson_symmetry': 'Pmmm', 'point_group': '222'}, 'P2_13': {'enc': '03bDODcODDdOOO0', 'full_symbol': 'P2_13', 'int_number': 198, 'order': 12, 'patterson_symmetry': 'Pm-3', 'point_group': '23'}, 'P3': {'enc': '01nOOO0', 'full_symbol': 'P3', 'int_number': 143, 'order': 3, 'patterson_symmetry': 'P-3', 'point_group': '3'}, 'P312': {'enc': '02nOOOfOOO0', 'full_symbol': 'P312', 'int_number': 149, 'order': 6, 'patterson_symmetry': 'P-31m', 'point_group': '32'}, 'P31c': {'enc': '02nOOOlOOD0', 'full_symbol': 'P31c', 'int_number': 159, 'order': 6, 'patterson_symmetry': 'P-31m', 'point_group': '3m'}, 'P31m': {'enc': '02nOOOlOOO0', 'full_symbol': 'P31m', 'int_number': 157, 'order': 6, 'patterson_symmetry': 'P-31m', 'point_group': '3m'}, 'P321': {'enc': '02nOOOeOOO0', 'full_symbol': 'P321', 'int_number': 150, 'order': 6, 'patterson_symmetry': 'P-3m1', 'point_group': '32'}, 'P3_1': {'enc': '01nOOC0', 'full_symbol': 'P3_1', 'int_number': 144, 'order': 3, 'patterson_symmetry': 'P-3', 'point_group': '3'}, 'P3_112': {'enc': '02nOOCfOOE0', 'full_symbol': 'P3_112', 'int_number': 151, 'order': 6, 'patterson_symmetry': 'P-31m', 'point_group': '32'}, 'P3_121': {'enc': '02nOOCeOOO0', 'full_symbol': 'P3_121', 'int_number': 152, 'order': 6, 'patterson_symmetry': 'P-3m1', 'point_group': '32'}, 'P3_2': {'enc': '01nOOE0', 'full_symbol': 'P3_2', 'int_number': 145, 'order': 3, 'patterson_symmetry': 'P-3', 'point_group': '3'}, 'P3_212': {'enc': '02nOOEfOOC0', 'full_symbol': 'P3_212', 'int_number': 153, 'order': 6, 'patterson_symmetry': 'P-31m', 'point_group': '32'}, 'P3_221': {'enc': '02nOOEeOOO0', 'full_symbol': 'P3_221', 'int_number': 154, 'order': 6, 'patterson_symmetry': 'P-3m1', 'point_group': '32'}, 'P3c1': {'enc': '02nOOOkOOD0', 'full_symbol': 'P3c1', 'int_number': 158, 'order': 6, 'patterson_symmetry': 'P-3m1', 'point_group': '3m'}, 'P3m1': {'enc': '02nOOOkOOO0', 'full_symbol': 'P3m1', 'int_number': 156, 'order': 6, 'patterson_symmetry': 'P-3m1', 'point_group': '3m'}, 'P4': {'enc': '02bOOOgOOO0', 'full_symbol': 'P4', 'int_number': 75, 'order': 4, 'patterson_symmetry': 'P4/m', 'point_group': '4'}, 'P4/m': {'enc': '12bOOOgOOO0', 'full_symbol': 'P4/m', 'int_number': 83, 'order': 8, 'patterson_symmetry': 'P4/m', 'point_group': '4/m'}, 'P4/mbm': {'enc': '13bOOOgOOOcDDO0', 'full_symbol': 'P4/m2_1/b2/m', 'int_number': 127, 'order': 16, 'patterson_symmetry': 'P4/mmm', 'point_group': '4/mmm'}, 'P4/mcc': {'enc': '13bOOOgOOOcOOD0', 'full_symbol': 'P4/m2/c2/c', 'int_number': 124, 'order': 16, 'patterson_symmetry': 'P4/mmm', 'point_group': '4/mmm'}, 'P4/mmm': {'enc': '13bOOOgOOOcOOO0', 'full_symbol': 'P4/m2/m2/m', 'int_number': 123, 'order': 16, 'patterson_symmetry': 'P4/mmm', 'point_group': '4/mmm'}, 'P4/mnc': {'enc': '13bOOOgOOOcDDD0', 'full_symbol': 'P4/m2_1/n2/c', 'int_number': 128, 'order': 16, 'patterson_symmetry': 'P4/mmm', 'point_group': '4/mmm'}, 'P4/n': {'enc': '03bOOOgDDOhDDO1YBO', 'full_symbol': 'P4/n', 'int_number': 85, 'order': 8, 'patterson_symmetry': 'P4/m', 'point_group': '4/m'}, 'P4/nbm': {'enc': '04bOOOgOOOcOOOhDDO1YYO', 'full_symbol': 'P4/n2/b2/m', 'int_number': 125, 'order': 16, 'patterson_symmetry': 'P4/mmm', 'point_group': '4/mmm'}, 'P4/ncc': {'enc': '04bOOOgDDOcDDDhDDO1YBO', 'full_symbol': 'P4/n2_1/c2/c', 'int_number': 130, 'order': 16, 'patterson_symmetry': 'P4/mmm', 'point_group': '4/mmm'}, 'P4/nmm': {'enc': '04bOOOgDDOcDDOhDDO1YBO', 'full_symbol': 'P4/n2_1/m2/m', 'int_number': 129, 'order': 16, 'patterson_symmetry': 'P4/mmm', 'point_group': '4/mmm'}, 'P4/nnc': {'enc': '04bOOOgOOOcOOOhDDD1YYY', 'full_symbol': 'P4/n2/n2/c', 'int_number': 126, 'order': 16, 'patterson_symmetry': 'P4/mmm', 'point_group': '4/mmm'}, 'P422': {'enc': '03bOOOgOOOcOOO0', 'full_symbol': 'P422', 'int_number': 89, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '422'}, 'P42_12': {'enc': '03bOOOgDDOcDDO0', 'full_symbol': 'P42_12', 'int_number': 90, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '422'}, 'P432': {'enc': '04bOOOcOOOdOOOeOOO0', 'full_symbol': 'P432', 'int_number': 207, 'order': 24, 'patterson_symmetry': 'Pm-3m', 'point_group': '432'}, 'P4_1': {'enc': '02bOODgOOB0', 'full_symbol': 'P4_1', 'int_number': 76, 'order': 4, 'patterson_symmetry': 'P4/m', 'point_group': '4'}, 'P4_122': {'enc': '03bOODgOOBcOOO0', 'full_symbol': 'P4_122', 'int_number': 91, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '422'}, 'P4_12_12': {'enc': '03bOODgDDBcDDB0', 'full_symbol': 'P4_12_12', 'int_number': 92, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '422'}, 'P4_132': {'enc': '04bDODcODDdOOOeFBB0', 'full_symbol': 'P4_132', 'int_number': 213, 'order': 24, 'patterson_symmetry': 'Pm-3m', 'point_group': '432'}, 'P4_2': {'enc': '02bOOOgOOD0', 'full_symbol': 'P4_2', 'int_number': 77, 'order': 4, 'patterson_symmetry': 'P4/m', 'point_group': '4'}, 'P4_2/m': {'enc': '12bOOOgOOD0', 'full_symbol': 'P4_2/m', 'int_number': 84, 'order': 8, 'patterson_symmetry': 'P4/m', 'point_group': '4/m'}, 'P4_2/mbc': {'enc': '13bOOOgOODcDDO0', 'full_symbol': 'P4_2/m2_1/b2/c', 'int_number': 135, 'order': 16, 'patterson_symmetry': 'P4/mmm', 'point_group': '4/mmm'}, 'P4_2/mcm': {'enc': '13bOOOgOODcOOD0', 'full_symbol': 'P4_2/m2/c2/m', 'int_number': 132, 'order': 16, 'patterson_symmetry': 'P4/mmm', 'point_group': '4/mmm'}, 'P4_2/mmc': {'enc': '13bOOOgOODcOOO0', 'full_symbol': 'P4_2/m2/m2/c', 'int_number': 131, 'order': 16, 'patterson_symmetry': 'P4/mmm', 'point_group': '4/mmm'}, 'P4_2/mnm': {'enc': '13bOOOgDDDcDDD0', 'full_symbol': 'P4_2/m2_1/n2/m', 'int_number': 136, 'order': 16, 'patterson_symmetry': 'P4/mmm', 'point_group': '4/mmm'}, 'P4_2/n': {'enc': '03bOOOgDDDhDDD1YYY', 'full_symbol': 'P4_2/n', 'int_number': 86, 'order': 8, 'patterson_symmetry': 'P4/m', 'point_group': '4/m'}, 'P4_2/nbc': {'enc': '04bOOOgDDDcOODhDDD1YBY', 'full_symbol': 'P4_2/n2/b2/c', 'int_number': 133, 'order': 16, 'patterson_symmetry': 'P4/mmm', 'point_group': '4/mmm'}, 'P4_2/ncm': {'enc': '04bOOOgDDDcDDOhDDD1YBY', 'full_symbol': 'P4_2/n2_1/c2/m', 'int_number': 138, 'order': 16, 'patterson_symmetry': 'P4/mmm', 'point_group': '4/mmm'}, 'P4_2/nmc': {'enc': '04bOOOgDDDcDDDhDDD1YBY', 'full_symbol': 'P4_2/n2_1/m2/c', 'int_number': 137, 'order': 16, 'patterson_symmetry': 'P4/mmm', 'point_group': '4/mmm'}, 'P4_2/nnm': {'enc': '04bOOOgDDDcOOOhDDD1YBY', 'full_symbol': 'P4_2/n2/n2/m', 'int_number': 134, 'order': 16, 'patterson_symmetry': 'P4/mmm', 'point_group': '4/mmm'}, 'P4_222': {'enc': '03bOOOgOODcOOO0', 'full_symbol': 'P4_222', 'int_number': 93, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '422'}, 'P4_22_12': {'enc': '03bOOOgDDDcDDD0', 'full_symbol': 'P4_22_12', 'int_number': 94, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '422'}, 'P4_232': {'enc': '04bOOOcOOOdOOOeDDD0', 'full_symbol': 'P4_232', 'int_number': 208, 'order': 24, 'patterson_symmetry': 'Pm-3m', 'point_group': '432'}, 'P4_2bc': {'enc': '03bOOOgOODjDDO0', 'full_symbol': 'P4_2bc', 'int_number': 106, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '4mm'}, 'P4_2cm': {'enc': '03bOOOgOODjOOD0', 'full_symbol': 'P4_2cm', 'int_number': 101, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '4mm'}, 'P4_2mc': {'enc': '03bOOOgOODjOOO0', 'full_symbol': 'P4_2mc', 'int_number': 105, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '4mm'}, 'P4_2nm': {'enc': '03bOOOgDDDjDDD0', 'full_symbol': 'P4_2nm', 'int_number': 102, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '4mm'}, 'P4_3': {'enc': '02bOODgOOF0', 'full_symbol': 'P4_3', 'int_number': 78, 'order': 4, 'patterson_symmetry': 'P4/m', 'point_group': '4'}, 'P4_322': {'enc': '03bOODgOOFcOOO0', 'full_symbol': 'P4_322', 'int_number': 95, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '422'}, 'P4_32_12': {'enc': '03bOODgDDFcDDF0', 'full_symbol': 'P4_32_12', 'int_number': 96, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '422'}, 'P4_332': {'enc': '04bDODcODDdOOOeBFF0', 'full_symbol': 'P4_332', 'int_number': 212, 'order': 24, 'patterson_symmetry': 'Pm-3m', 'point_group': '432'}, 'P4bm': {'enc': '03bOOOgOOOjDDO0', 'full_symbol': 'P4bm', 'int_number': 100, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '4mm'}, 'P4cc': {'enc': '03bOOOgOOOjOOD0', 'full_symbol': 'P4cc', 'int_number': 103, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '4mm'}, 'P4mm': {'enc': '03bOOOgOOOjOOO0', 'full_symbol': 'P4mm', 'int_number': 99, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '4mm'}, 'P4nc': {'enc': '03bOOOgOOOjDDD0', 'full_symbol': 'P4nc', 'int_number': 104, 'order': 8, 'patterson_symmetry': 'P4/mmm', 'point_group': '4mm'}, 'P6': {'enc': '02nOOObOOO0', 'full_symbol': 'P6', 'int_number': 168, 'order': 6, 'patterson_symmetry': 'P6/m', 'point_group': '6'}, 'P6/m': {'enc': '12nOOObOOO0', 'full_symbol': 'P6/m', 'int_number': 175, 'order': 12, 'patterson_symmetry': 'P6/m', 'point_group': '6/m'}, 'P6/mcc': {'enc': '13nOOObOOOeOOD0', 'full_symbol': 'P6/m2/c2/c', 'int_number': 192, 'order': 24, 'patterson_symmetry': 'P6/mmm', 'point_group': '6/mmm'}, 'P6/mmm': {'enc': '13nOOObOOOeOOO0', 'full_symbol': 'P6/m2/m2/m', 'int_number': 191, 'order': 24, 'patterson_symmetry': 'P6/mmm', 'point_group': '6/mmm'}, 'P622': {'enc': '03nOOObOOOeOOO0', 'full_symbol': 'P622', 'int_number': 177, 'order': 12, 'patterson_symmetry': 'P6/mmm', 'point_group': '622'}, 'P6_1': {'enc': '02nOOCbOOD0', 'full_symbol': 'P6_1', 'int_number': 169, 'order': 6, 'patterson_symmetry': 'P6/m', 'point_group': '6'}, 'P6_122': {'enc': '03nOOCbOODeOOC0', 'full_symbol': 'P6_122', 'int_number': 178, 'order': 12, 'patterson_symmetry': 'P6/mmm', 'point_group': '622'}, 'P6_2': {'enc': '02nOOEbOOO0', 'full_symbol': 'P6_2', 'int_number': 171, 'order': 6, 'patterson_symmetry': 'P6/m', 'point_group': '6'}, 'P6_222': {'enc': '03nOOEbOOOeOOE0', 'full_symbol': 'P6_222', 'int_number': 180, 'order': 12, 'patterson_symmetry': 'P6/mmm', 'point_group': '622'}, 'P6_3': {'enc': '02nOOObOOD0', 'full_symbol': 'P6_3', 'int_number': 173, 'order': 6, 'patterson_symmetry': 'P6/m', 'point_group': '6'}, 'P6_3/m': {'enc': '12nOOObOOD0', 'full_symbol': 'P6_3/m', 'int_number': 176, 'order': 12, 'patterson_symmetry': 'P6/m', 'point_group': '6/m'}, 'P6_3/mcm': {'enc': '13nOOObOODeOOD0', 'full_symbol': 'P6_3/m2/c2/m', 'int_number': 193, 'order': 24, 'patterson_symmetry': 'P6/mmm', 'point_group': '6/mmm'}, 'P6_3/mmc': {'enc': '13nOOObOODeOOO0', 'full_symbol': 'P6_3/m2/m2/c', 'int_number': 194, 'order': 24, 'patterson_symmetry': 'P6/mmm', 'point_group': '6/mmm'}, 'P6_322': {'enc': '03nOOObOODeOOO0', 'full_symbol': 'P6_322', 'int_number': 182, 'order': 12, 'patterson_symmetry': 'P6/mmm', 'point_group': '622'}, 'P6_3cm': {'enc': '03nOOObOODkOOD0', 'full_symbol': 'P6_3cm', 'int_number': 185, 'order': 12, 'patterson_symmetry': 'P6/mmm', 'point_group': '6mm'}, 'P6_3mc': {'enc': '03nOOObOODkOOO0', 'full_symbol': 'P6_3mc', 'int_number': 186, 'order': 12, 'patterson_symmetry': 'P6/mmm', 'point_group': '6mm'}, 'P6_4': {'enc': '02nOOCbOOO0', 'full_symbol': 'P6_4', 'int_number': 172, 'order': 6, 'patterson_symmetry': 'P6/m', 'point_group': '6'}, 'P6_422': {'enc': '03nOOCbOOOeOOC0', 'full_symbol': 'P6_422', 'int_number': 181, 'order': 12, 'patterson_symmetry': 'P6/mmm', 'point_group': '622'}, 'P6_5': {'enc': '02nOOEbOOD0', 'full_symbol': 'P6_5', 'int_number': 170, 'order': 6, 'patterson_symmetry': 'P6/m', 'point_group': '6'}, 'P6_522': {'enc': '03nOOEbOODeOOE0', 'full_symbol': 'P6_522', 'int_number': 179, 'order': 12, 'patterson_symmetry': 'P6/mmm', 'point_group': '622'}, 'P6cc': {'enc': '03nOOObOOOkOOD0', 'full_symbol': 'P6cc', 'int_number': 184, 'order': 12, 'patterson_symmetry': 'P6/mmm', 'point_group': '6mm'}, 'P6mm': {'enc': '03nOOObOOOkOOO0', 'full_symbol': 'P6mm', 'int_number': 183, 'order': 12, 'patterson_symmetry': 'P6/mmm', 'point_group': '6mm'}, 'Pa-3': {'enc': '13bDODcODDdOOO0', 'full_symbol': 'P2_1/a-3', 'int_number': 205, 'order': 24, 'patterson_symmetry': 'Pm-3', 'point_group': 'm-3'}, 'Pba2': {'enc': '02bOOOjDDO0', 'full_symbol': 'Pba2', 'int_number': 32, 'order': 4, 'patterson_symmetry': 'Pmmm', 'point_group': 'mm2'}, 'Pbam': {'enc': '12bOOOcDDO0', 'full_symbol': 'P2_1/b2_1/a2/m', 'int_number': 55, 'order': 8, 'patterson_symmetry': 'Pmmm', 'point_group': 'mmm'}, 'Pban': {'enc': '03bOOOcOOOhDDO1BBO', 'full_symbol': 'P2/b2/a2/n', 'int_number': 50, 'order': 8, 'patterson_symmetry': 'Pmmm', 'point_group': 'mmm'}, 'Pbca': {'enc': '12bDODcODD0', 'full_symbol': 'P2_1/b2_1/c2_1/a', 'int_number': 61, 'order': 8, 'patterson_symmetry': 'Pmmm', 'point_group': 'mmm'}, 'Pbcm': {'enc': '12bOODcODD0', 'full_symbol': 'P2/b2_1/c2_1/m', 'int_number': 57, 'order': 8, 'patterson_symmetry': 'Pmmm', 'point_group': 'mmm'}, 'Pbcn': {'enc': '12bDDDcOOD0', 'full_symbol': 'P2_1/b2/c2_1/n', 'int_number': 60, 'order': 8, 'patterson_symmetry': 'Pmmm', 'point_group': 'mmm'}, 'Pca2_1': {'enc': '02bOODjDOO0', 'full_symbol': 'Pca2_1', 'int_number': 29, 'order': 4, 'patterson_symmetry': 'Pmmm', 'point_group': 'mm2'}, 'Pcc2': {'enc': '02bOOOjOOD0', 'full_symbol': 'Pcc2', 'int_number': 27, 'order': 4, 'patterson_symmetry': 'Pmmm', 'point_group': 'mm2'}, 'Pcca': {'enc': '12bDOOcOOD0', 'full_symbol': 'P2_1/c2/c2/a', 'int_number': 54, 'order': 8, 'patterson_symmetry': 'Pmmm', 'point_group': 'mmm'}, 'Pccm': {'enc': '12bOOOcOOD0', 'full_symbol': 'P2/c2/c2/m', 'int_number': 49, 'order': 8, 'patterson_symmetry': 'Pmmm', 'point_group': 'mmm'}, 'Pccn': {'enc': '12bDDOcODD0', 'full_symbol': 'P2_1/c2_1/c2/n', 'int_number': 56, 'order': 8, 'patterson_symmetry': 'Pmmm', 'point_group': 'mmm'}, 'Pm-3': {'enc': '13bOOOcOOOdOOO0', 'full_symbol': 'P2/m-3', 'int_number': 200, 'order': 24, 'patterson_symmetry': 'Pm-3', 'point_group': 'm-3'}, 'Pm-3m': {'enc': '14bOOOcOOOdOOOeOOO0', 'full_symbol': 'P4/m-32/m', 'int_number': 221, 'order': 48, 'patterson_symmetry': 'Pm-3m', 'point_group': 'm-3m'}, 'Pm-3n': {'enc': '14bOOOcOOOdOOOeDDD0', 'full_symbol': 'P4_2/m-32/n', 'int_number': 223, 'order': 48, 'patterson_symmetry': 'Pm-3m', 'point_group': 'm-3m'}, 'Pma2': {'enc': '02bOOOjDOO0', 'full_symbol': 'Pma2', 'int_number': 28, 'order': 4, 'patterson_symmetry': 'Pmmm', 'point_group': 'mm2'}, 'Pmc2_1': {'enc': '02bOODjOOD0', 'full_symbol': 'Pmc2_1', 'int_number': 26, 'order': 4, 'patterson_symmetry': 'Pmmm', 'point_group': 'mm2'}, 'Pmm2': {'enc': '02bOOOjOOO0', 'full_symbol': 'Pmm2', 'int_number': 25, 'order': 4, 'patterson_symmetry': 'Pmmm', 'point_group': 'mm2'}, 'Pmma': {'enc': '12bDOOcOOO0', 'full_symbol': 'P2_1/m2/m2/a', 'int_number': 51, 'order': 8, 'patterson_symmetry': 'Pmmm', 'point_group': 'mmm'}, 'Pmmm': {'enc': '12bOOOcOOO0', 'full_symbol': 'P2/m2/m2/m', 'int_number': 47, 'order': 8, 'patterson_symmetry': 'Pmmm', 'point_group': 'mmm'}, 'Pmmn': {'enc': '03bOOOcDDOhDDO1BBO', 'full_symbol': 'P2_1/m2_1/m2/n', 'int_number': 59, 'order': 8, 'patterson_symmetry': 'Pmmm', 'point_group': 'mmm'}, 'Pmn2_1': {'enc': '02bDODjDOD0', 'full_symbol': 'Pmn2_1', 'int_number': 31, 'order': 4, 'patterson_symmetry': 'Pmmm', 'point_group': 'mm2'}, 'Pmna': {'enc': '12bDODcDOD0', 'full_symbol': 'P2/m2/n2_1/a', 'int_number': 53, 'order': 8, 'patterson_symmetry': 'Pmmm', 'point_group': 'mmm'}, 'Pn-3': {'enc': '04bOOOcOOOdOOOhDDD1YYY', 'full_symbol': 'P2/n-3', 'int_number': 201, 'order': 24, 'patterson_symmetry': 'Pm-3', 'point_group': 'm-3'}, 'Pn-3m': {'enc': '05bOOOcOOOdOOOeDDDhDDD1YYY', 'full_symbol': 'P4_2/n-32/m', 'int_number': 224, 'order': 48, 'patterson_symmetry': 'Pm-3m', 'point_group': 'm-3m'}, 'Pn-3n': {'enc': '05bOOOcOOOdOOOeOOOhDDD1YYY', 'full_symbol': 'P4/n-32/n', 'int_number': 222, 'order': 48, 'patterson_symmetry': 'Pm-3m', 'point_group': 'm-3m'}, 'Pna2_1': {'enc': '02bOODjDDO0', 'full_symbol': 'Pna2_1', 'int_number': 33, 'order': 4, 'patterson_symmetry': 'Pmmm', 'point_group': 'mm2'}, 'Pnc2': {'enc': '02bOOOjODD0', 'full_symbol': 'Pnc2', 'int_number': 30, 'order': 4, 'patterson_symmetry': 'Pmmm', 'point_group': 'mm2'}, 'Pnma': {'enc': '12bDODcODO0', 'full_symbol': 'P2_1/n2_1/m2_1/a', 'int_number': 62, 'order': 8, 'patterson_symmetry': 'Pmmm', 'point_group': 'mmm'}, 'Pnn2': {'enc': '02bOOOjDDD0', 'full_symbol': 'Pnn2', 'int_number': 34, 'order': 4, 'patterson_symmetry': 'Pmmm', 'point_group': 'mm2'}, 'Pnna': {'enc': '12bDOOcDDD0', 'full_symbol': 'P2/n2_1/n2/a', 'int_number': 52, 'order': 8, 'patterson_symmetry': 'Pmmm', 'point_group': 'mmm'}, 'Pnnm': {'enc': '12bOOOcDDD0', 'full_symbol': 'P2_1/n2_1/n2/m', 'int_number': 58, 'order': 8, 'patterson_symmetry': 'Pmmm', 'point_group': 'mmm'}, 'Pnnn': {'enc': '03bOOOcOOOhDDD1BBB', 'full_symbol': 'P2/n2/n2/n', 'int_number': 48, 'order': 8, 'patterson_symmetry': 'Pmmm', 'point_group': 'mmm'}, 'R-3': {'enc': '12aECCnOOO0', 'full_symbol': 'R-3', 'int_number': 148, 'order': 18, 'patterson_symmetry': 'R-3', 'point_group': '-3'}, 'R-3c': {'enc': '13aECCnOOOeOOD0', 'full_symbol': 'R-32/c', 'int_number': 167, 'order': 36, 'patterson_symmetry': 'R-3m', 'point_group': '-3m'}, 'R-3m': {'enc': '13aECCnOOOeOOO0', 'full_symbol': 'R-32/m', 'int_number': 166, 'order': 36, 'patterson_symmetry': 'R-3m', 'point_group': '-3m'}, 'R3': {'enc': '02aECCnOOO0', 'full_symbol': 'R3', 'int_number': 146, 'order': 9, 'patterson_symmetry': 'R-3', 'point_group': '3'}, 'R32': {'enc': '03aECCnOOOeOOO0', 'full_symbol': 'R32', 'int_number': 155, 'order': 18, 'patterson_symmetry': 'R-3m', 'point_group': '32'}, 'R3c': {'enc': '03aECCnOOOkOOD0', 'full_symbol': 'R3c', 'int_number': 161, 'order': 18, 'patterson_symmetry': 'R-3m', 'point_group': '3m'}, 'R3m': {'enc': '03aECCnOOOkOOO0', 'full_symbol': 'R3m', 'int_number': 160, 'order': 18, 'patterson_symmetry': 'R-3m', 'point_group': '3m'}}

def tokenize_enc(enc_string):
    """
    Tokenizes the 'enc' string based on the following rules:
    - First digit: 0->o, 1->+
    - Second digit: 0->z, 1->y, 2->x, etc.
    - Last digit if it's 0: 0->o
    - If fourth from last is 1: 1->+
    - Three consecutive uppercase letters or three consecutive digits are treated as one token
    - Existing lowercase letters are preserved
    
    Args:
        enc_string (str): The 'enc' string to tokenize.
    
    Returns:
        str: The tokenized string with tokens separated by spaces.
    """
    # Convert string to list for easier manipulation
    chars = list(enc_string)
    
    # Handle first digit (0->o, 1->+)
    if chars[0] == '0':
        chars[0] = 'o'
    elif chars[0] == '1':
        chars[0] = '+'
    
    # Handle second digit (0->z, 1->y, etc) only if it's a digit
    if len(chars) > 1 and chars[1].isdigit():
        chars[1] = chr(ord('z') - int(chars[1]))
    
    # Handle last digit if it's 0
    if chars[-1] == '0':
        chars[-1] = 'o'
    
    # Handle fourth from last if it's 1
    if len(chars) >= 4 and chars[-4] == '1':
        chars[-4] = '+'
    
    # Join back to string
    converted_string = ''.join(chars)
    
    # Regular expression to match three uppercase letters or three consecutive digits
    pattern = re.compile(r'[A-Z]{3}|.')
    
    tokens = pattern.findall(converted_string)
    
    # Filter out any empty strings (if any)
    tokens = [token for token in tokens if token]
    
    # Join tokens with space
    tokenized = ' '.join(tokens)
    
    return tokenized

def get_tokenized_enc(int_number):
    """
    Given an int_number, finds the corresponding 'enc' string in sg_encoding,
    tokenizes it, and returns the tokenized version.
    
    Args:
        int_number (int): The integer number to search for.
    
    Returns:
        str: The tokenized 'enc' string.
    
    Raises:
        ValueError: If the int_number is not found in sg_encoding.
    """
    for symbol, properties in sg_encoding.items():
        if properties.get('int_number') == int_number:
            enc_string = properties.get('enc')
            tokenized = tokenize_enc(enc_string)
            return tokenized
    raise ValueError(f"int_number {int_number} not found in sg_encoding.")
    
def get_space_group_num(enc):
    """
    Given an 'enc' string, finds the corresponding 'int_number' in sg_encoding.

    Args:
        enc (str): The 'enc' string to search for.

    Returns:
        int: The corresponding 'int_number'.

    Raises:
        ValueError: If the 'enc' string is not found in sg_encoding.
    """
    for symbol, properties in sg_encoding.items():
        if properties.get('enc') == enc:
            int_number = properties.get('int_number')
            return int_number
    raise ValueError(f"enc {enc} not found in sg_encoding.")

def get_space_group_num_from_letter_enc(letter_enc):
    """
    Given a letter encoded string, converts it back to number encoding and returns the space group number.
    Reverse mapping:
    - First letter: o->0, +->1
    - Second letter: z->0, y->1, x->2, etc.
    - Last letter if o: o->0
    - Fourth from last if +: +->1
    
    Args:
        letter_enc (str): The letter encoded string without spaces.
    
    Returns:
        int: The corresponding space group number.
    
    Raises:
        ValueError: If the encoding is not found in sg_encoding.
    """
    # Convert string to list for easier manipulation
    chars = list(letter_enc)
    
    # Handle first letter
    if chars[0] == 'o':
        chars[0] = '0'
    elif chars[0] == '+':
        chars[0] = '1'
    
    # Handle second letter
    if len(chars) > 1 and chars[1].islower():
        chars[1] = str(ord('z') - ord(chars[1]))
    
    # Handle last letter if it's o
    if chars[-1] == 'o':
        chars[-1] = '0'
    
    # Handle fourth from last if it's +
    if len(chars) >= 4 and chars[-4] == '+':
        chars[-4] = '1'
    
    number_enc = ''.join(chars)
    return get_space_group_num(number_enc)


